@@ -1,18 +1,48 @@
-__version__ = '1.1'
+__version__ = '1.2'
 
 from redis import Redis
 import pyres.json_parser as json
 
+import os
 import time, datetime
 import sys
 import logging
 
-def setup_logging(log_level=logging.INFO, filename=None, stream=sys.stderr):
-    if log_level == logging.NOTSET:
-        return
-    logger = logging.getLogger()
-    logger.setLevel(log_level)
-    if filename:
+def special_log_file(filename):
+    if filename in ("stderr", "stdout"):
+        return True
+    if filename.startswith("syslog"):
+        return True
+    return False
+
+def get_logging_handler(filename, procname, namespace=None):
+    if namespace:
+        message_format = namespace + ': %(message)s'
+    else:
+        message_format = '%(message)s'
+    format = '%(asctime)s %(levelname)-8s ' + message_format
+
+    if not filename:
+        filename = "stderr"
+    if filename == "stderr":
+        handler = logging.StreamHandler(sys.stderr)
+    elif filename == "stdout":
+        handler = logging.StreamHandler(sys.stdout)
+    elif filename.startswith("syslog"): # "syslog:local0"
+        from logging.handlers import SysLogHandler
+        facility_name = filename[7:] or 'user'
+        facility = SysLogHandler.facility_names[facility_name]
+
+        if os.path.exists("/dev/log"):
+            syslog_path = "/dev/log"
+        elif os.path.exists("/var/run/syslog"):
+            syslog_path = "/var/run/syslog"
+        else:
+            raise Exception("Unable to figure out the syslog socket path")
+
+        handler = SysLogHandler(syslog_path, facility)
+        format = procname + "[%(process)d]: " + message_format
+    else:
         try:
             from logging.handlers import WatchedFileHandler
             handler = WatchedFileHandler(filename)
@@ -20,12 +50,26 @@ def setup_logging(log_level=logging.INFO, filename=None, stream=sys.stderr):
             from logging.handlers import RotatingFileHandler
             handler = RotatingFileHandler(filename,maxBytes=52428800,
                                           backupCount=7)
-    else:
-        handler = logging.StreamHandler(stream)
-    handler.setFormatter(logging.Formatter(
-        '%(asctime)s %(levelname)-8s %(message)s', '%Y-%m-%d %H:%M:%S'))
+    handler.setFormatter(logging.Formatter(format, '%Y-%m-%d %H:%M:%S'))
+    return handler
+
+def setup_logging(procname, log_level=logging.INFO, filename=None):
+    if log_level == logging.NOTSET:
+        return
+    logger = logging.getLogger()
+    logger.setLevel(log_level)
+    handler = get_logging_handler(filename, procname)
     logger.addHandler(handler)
 
+def setup_pidfile(path):
+    if not path:
+        return
+    dirname = os.path.dirname(path)
+    if dirname and not os.path.exists(dirname):
+        os.makedirs(dirname)
+    with open(path, 'w') as f:
+        f.write(str(os.getpid()))
+
 def my_import(name):
     """Helper function for walking import calls when searching for classes by
     string names.
@@ -105,13 +149,16 @@ class ResQ(object):
         self.watch_queue(queue)
         self.redis.rpush("resque:queue:%s" % queue, ResQ.encode(item))
 
-    def pop(self, queue, timeout=10):
-        ret = self.redis.blpop("resque:queue:%s" % queue, timeout=timeout)
+    def pop(self, queues, timeout=10):
+        if isinstance(queues, basestring):
+            queues = [queues]
+        ret = self.redis.blpop(["resque:queue:%s" % q for q in queues],
+                               timeout=timeout)
         if ret:
-            if isinstance(ret, tuple):
-                q, ret = ret
-            return ResQ.decode(ret)
-        return ret
+            key, ret = ret
+            return key[13:], ResQ.decode(ret)  # trim "resque:queue:"
+        else:
+            return None, None
 
     def size(self, queue):
         return int(self.redis.llen("resque:queue:%s" % queue))
@@ -166,7 +213,7 @@ class ResQ(object):
         if queue:
             class_name = '%s.%s' % (klass.__module__, klass.__name__)
             self.push(queue, {'class':class_name,'args':args})
-            logging.info("enqueued '%s' job" % class_name)
+            logging.info("enqueued '%s' job on queue %s" % (class_name, queue))
             if args:
                 logging.debug("job arguments: %s" % str(args))
             else:
@@ -179,7 +226,7 @@ class ResQ(object):
         if 'first_attempt' in kwargs:
             payload['first_attempt'] = kwargs['first_attempt']
         self.push(queue, payload)
-        logging.info("enqueued '%s' job" % klass_as_string)
+        logging.info("enqueued '%s' job on queue %s" % (klass_as_string, queue))
         if args:
             logging.debug("job arguments: %s" % str(args))
         else:
@@ -210,9 +257,9 @@ class ResQ(object):
         return [key.replace('resque:','')
                 for key in self.redis.keys('resque:*')]
 
-    def reserve(self, queue):
+    def reserve(self, queues):
         from pyres.job import Job
-        return Job.reserve(queue, self)
+        return Job.reserve(queues, self)
 
     def __str__(self):
         return "PyRes Client connected to %s" % self.redis.server
@@ -235,12 +282,12 @@ class ResQ(object):
         """Close the underlying redis connection.
 
         """
-        self.redis.connection.disconnect()
+        self.redis.connection_pool.get_connection('_').disconnect()
 
     def enqueue_at(self, datetime, klass, *args, **kwargs):
         class_name = '%s.%s' % (klass.__module__, klass.__name__)
-        logging.info("enqueued '%s' job for execution at %s" % (class_name,
-                                                                datetime))
+        logging.info("scheduled '%s' job on queue %s for execution at %s" %
+                     (class_name, klass.queue, datetime))
         if args:
             logging.debug("job arguments are: %s" % str(args))
         payload = {'class':class_name, 'queue': klass.queue, 'args':args}
